@@ -9,19 +9,31 @@
 #' @param data A \code{data.frame} with the source data
 #' @param blocks An optional specification for blocks of variables in
 #' the rows. The default assigns each variable in its own block.
+#' @param predictorMatrix A predictor matrix from which rows with the same
+#' names are copied into the output predictor matrix.
 #' @return A matrix
 #' @seealso \code{\link{make.blocks}}
 #' @examples
 #' make.predictorMatrix(nhanes)
 #' make.predictorMatrix(nhanes, blocks = make.blocks(nhanes, "collect"))
 #' @export
-make.predictorMatrix <- function(data, blocks = make.blocks(data)) {
+make.predictorMatrix <- function(data, blocks = make.blocks(data),
+                                 predictorMatrix = NULL) {
+  input.predictorMatrix <- predictorMatrix
   data <- check.dataform(data)
   predictorMatrix <- matrix(1, nrow = length(blocks), ncol = ncol(data))
   dimnames(predictorMatrix) <- list(names(blocks), colnames(data))
   for (i in row.names(predictorMatrix)) {
     predictorMatrix[i, colnames(predictorMatrix) %in% i] <- 0
   }
+  # preserve any user setting in predictorMatrix specification
+  if (!is.null(input.predictorMatrix)) {
+    for (i in row.names(predictorMatrix)) {
+      if (i %in% row.names(input.predictorMatrix)) {
+        predictorMatrix[i, ] <- input.predictorMatrix[i, ]
+      }
+    }
+  }
   predictorMatrix
 }
 
@@ -40,11 +52,12 @@ check.predictorMatrix <- function(predictorMatrix,
   # if we have no blocks, restrict to square predictorMatrix
   if (is.null(blocks)) {
     if (nrow(predictorMatrix) != ncol(predictorMatrix)) {
-      stop(paste(
-        "If no blocks are specified, predictorMatrix must",
-        "have same number of rows and columns"
-      ),
-      call. = FALSE
+      stop(
+        paste(
+          "If no blocks are specified, predictorMatrix must",
+          "have same number of rows and columns"
+        ),
+        call. = FALSE
       )
     }
     if (is.null(dimnames(predictorMatrix))) {
@@ -62,17 +75,18 @@ check.predictorMatrix <- function(predictorMatrix,
 
   # check conforming arguments
   if (nrow(predictorMatrix) > length(blocks)) {
-    stop(paste0(
-      "predictorMatrix has more rows (", nrow(predictorMatrix),
-      ") than blocks (", length(blocks), ")"
-    ),
-    call. = FALSE
+    stop(
+      paste0(
+        "predictorMatrix has more rows (", nrow(predictorMatrix),
+        ") than blocks (", length(blocks), ")"
+      ),
+      call. = FALSE
     )
   }
 
   # borrow rownames from blocks if needed
   if (is.null(rownames(predictorMatrix)) &&
-    nrow(predictorMatrix) == length(blocks)) {
+      nrow(predictorMatrix) == length(blocks)) {
     rownames(predictorMatrix) <- names(blocks)
   }
   if (is.null(rownames(predictorMatrix))) {
@@ -81,7 +95,7 @@ check.predictorMatrix <- function(predictorMatrix,
 
   # borrow blocknames from predictorMatrix if needed
   if (is.null(names(blocks)) &&
-    nrow(predictorMatrix) == length(blocks)) {
+      nrow(predictorMatrix) == length(blocks)) {
     names(blocks) <- rownames(predictorMatrix)
   }
   if (is.null(names(blocks))) {
@@ -92,14 +106,14 @@ check.predictorMatrix <- function(predictorMatrix,
   found <- rownames(predictorMatrix) %in% names(blocks)
   if (!all(found)) {
     stop("Names not found in blocks: ",
-      paste(rownames(predictorMatrix)[!found], collapse = ", "),
-      call. = FALSE
+         paste(rownames(predictorMatrix)[!found], collapse = ", "),
+         call. = FALSE
     )
   }
 
   # borrow colnames from data if needed
   if (is.null(colnames(predictorMatrix)) &&
-    ncol(predictorMatrix) == ncol(data)) {
+      ncol(predictorMatrix) == ncol(data)) {
     colnames(predictorMatrix) <- names(data)
   }
   if (is.null(colnames(predictorMatrix))) {
@@ -110,8 +124,8 @@ check.predictorMatrix <- function(predictorMatrix,
   found <- colnames(predictorMatrix) %in% names(data)
   if (!all(found)) {
     stop("Names not found in data: ",
-      paste(colnames(predictorMatrix)[!found], collapse = ", "),
-      call. = FALSE
+         paste(colnames(predictorMatrix)[!found], collapse = ", "),
+         call. = FALSE
     )
   }
 
